# TP 16 (version couleurs) correction : Manipulation d'images
# BCPST1B 2024-2025
# Lycée Hoche, Versailles
# L.-C. LEFÈVRE

#%% modules et fonctions globales
# exécuter une fois pour toute, regarder, ne pas toucher

import numpy as np
import matplotlib.pyplot as plt
import PIL

def afficher(X):
    plt.imshow(X)
    plt.show()

# affiche les images X et Y les unes au-dessus des autres
def afficher2(X, Y):
    plt.subplot(2, 1, 1)
    plt.imshow(X)
    plt.subplot(2, 1, 2)
    plt.imshow(Y)
    plt.show()

def taille(X):
    (n, p, _) = X.shape
    return (n, p)

def image_vierge(n, p):
    Y = np.zeros((n, p, 3), dtype=np.uint8)
    return Y

def copie(X):
    Y = X.copy()
    return Y

# sauvegarde l'image X dans le fichier nommé nom
# par exemple sauvegarde(X, "nom.jpg")
def sauvegarde(X, nom):
    # f représente une image PIL
    f = PIL.Image.fromarray(X, mode="RGB")
    f.save(nom)

#%% paramètres utilisateurs globaux
# exécuter après chaque changement d'image

# choisir son image dans le dossier Photos
choix = "./Photos/Hoche.jpg"

# chargement de l'image et test de l'affichage
with PIL.Image.open(choix) as f:
    # f représente un fichier image PIL
    # g est l'image PIL convertie en couleurs
    g = f.convert("RGB")
    # image est un tableau numpy obtenu à partir de g
    image = np.asarray(g)
    # tests
    print("Image choisie :", choix)
    print("Mode (L : niveaux de gris, RGB : couleurs) :", g.mode)
    print("Dimensions du tableau :", image.shape)
    print("Type des données :", image.dtype)
    afficher(image)

#%% exercice 9

def contraste(X):
    (n, p) = taille(X)
    Y = image_vierge(n, p)
    for i in range(n):
        for j in range(p):
            # contraste chaque couleur
            for c in range(3):
                if X[i, j, c] < 96:
                    Y[i, j, c] =  1/4 * X[i, j, c]
                elif X[i, j, c] < 160:
                    Y[i, j, c] = 24 + 13/4 * (X[i, j, c] - 96)
                else:
                    Y[i, j, c] = 232 + 1/4 * (X[i, j, c] - 160)
    return Y

# test
Y = contraste(image)
afficher2(image, Y)

#%%

def negatif(X):
    (n, p) = taille(X)
    Y = image_vierge(n, p)
    for i in range(n):
        for j in range(p):
            # inverse chaque couleur
            for c in range(3):
                Y[i, j, c] = 255 -  X[i, j, c]
    return Y

# test
Y = negatif(image)
afficher(Y)

#%% exercice 10

def seuillage(X, a, b, c):
    (n, p) = taille(X)
    Y = image_vierge(n, p)
    for i in range(n):
        for j in range(p):
            # seuil a pour le rouge
            if X[i, j, 0] <= a:
                Y[i, j, 0] = 0
            else:
                Y[i, j, 0] = 255
            # seuil b pour le vert
            if X[i, j, 1] <= b:
                Y[i, j, 1] = 0
            else:
                Y[i, j, 1] = 255
            # seuil c pour bleu
            if X[i, j, 2] <= c:
                Y[i, j, 2] = 0
            else:
                Y[i, j, 2] = 255
    return Y

# test
Y = seuillage(image, 127, 127, 127)
afficher(Y)

#%%
# autre test
choix = "./Photos/Amsterdam.jpg"
with PIL.Image.open(choix) as f:
    g = f.convert("RGB")
    image_Amsterdam = np.asarray(g)

Y = seuillage(image_Amsterdam, 127, 127, 127)
afficher(Y)

#%%
# autre test
choix = "./Photos/chats.jpg"
with PIL.Image.open(choix) as f:
    g = f.convert("RGB")
    image_chats = np.asarray(g)

Y = seuillage(image_chats, 160, 127, 127)
afficher(Y)

#%% exercice 11

def fusion(X1, X2):
    (n, p) = taille(X1)
    (n2, p2) = taille(X2)
    assert n == n2 and p == p2, "Les images à fusionner doivent être exactement de la même taille"
    Y = image_vierge(n, p)
    for i in range(n):
        for j in range(p):
            Y[i, j] = 1/2 * X1[i, j] + 1/2 * X2[i, j]
    return Y

# test
choix1 = "./Photos/Hoche.jpg"
choix2 = "./Photos/Miami.jpg"
with PIL.Image.open(choix1) as f1, PIL.Image.open(choix2) as f2:
    g1 = f1.convert("RGB")
    X1 = np.asarray(g1)
    g2 = f2.convert("RGB")
    X2 = np.asarray(g2)

Y = fusion(X1, X2)
afficher(Y)
#%%
sauvegarde(Y, "fusion-Hoche-Miami.jpg")

#%%
# bonus
#%%
# la spéciale
choix1 = "./Photos/Hoche.jpg"
choix2 = "./Photos/surf.jpg"
with PIL.Image.open(choix1) as f1, PIL.Image.open(choix2) as f2:
    g1 = f1.convert("RGB")
    X1 = np.asarray(g1)
    g2 = f2.convert("RGB")
    X2 = np.asarray(g2)
Y = copie(X1)
# coordonnées du coin haut gauche où coller X2 dans X1
ux = 115
uy = 51
# coordonnées du coin haut gauche du rectangle de X2 à découper
vx = 115
vy = 50
# coordonnées du coin bas droit du rectangle de X2 à découper
tx = 298
ty = 250
for i in range(ux, ux + tx - vx):
    for j in range(uy, uy + ty - vy):
        Y[i, j] = 2/3 * X2[i - ux + vx, j - uy + vy] + 1/3 * X1[i, j]
afficher(Y)
#%%
sauvegarde(Y, "fusion-Hoche-surf.jpg")
